from django.template import Library, TemplateSyntaxError

from model_nodes import ModelDetailNode, ModelTeaserNode, ModelListNode

register = Library()

@register.tag
def detail_block(parser, token):
    """
    Template tag that takes a model instance and returns the given instance as 
    a template-formatted block.  Inserts two objects into the context:
      ``instance`` - The model instance
      ``fields`` - A list of (name, label, value) tuples representing the 
                   instance's fields
    
    You would want to use the tag instead of the filter primarily because, with
    a tag you can override the default template used for a particular model.
    For example, in the example project, Pepulators have Jambs, so when we put
    a detail block of a Pepulator on a page, a list of Jambs will show up.  Say
    we don't want the default detail of jambs to display; say we want a custom
    Jamb detail template.  Then we can say the following:
    
        {% with pepulator_factory_jamb_detail_template="pepulator_factory/jamb_detail.html" %}
            {% detail_block pepulator %}
        {% endwith %}
    
    The custom template is named by the app name (``pepulator_factory``), the 
    model name in all lowercase (``jamb``) and the suffix ``_template``.
    """
    try:
        tag_name, instance_name = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError("%r tag requires exactly two arguments" % 
                                  token.contents.split()[0])
    
    node = ModelDetailNode(instance_name, resolved=False)
    return node


@register.tag
def teaser_block(parser, token):
    """
    Template tag that takes a model instance and returns the given instance as 
    a template-formatted block.  Inserts two objects into the context:
      ``instance`` - The model instance
      ``fields`` - A list of (name, label, value) tuples representing the 
                   instance's fields
    
    You would want to use the tag instead of the filter primarily because, with
    a tag you can override the default template used for a particular model.
    For example, in the example project, Pepulators have Jambs, so when we put
    a detail block of a Pepulator on a page, a list of Jambs will show up.  Say
    we don't want the default detail of jambs to display; say we want a custom
    Jamb detail template.  Then we can say the following:
    
        {% with pepulator_factory_jamb_teaser_template="pepulator_factory/jamb_detail.html" %}
            {% teaser_block pepulator %}
        {% endwith %}
    
    The custom template is named by the app name (``pepulator_factory``), the 
    model name in all lowercase (``jamb``) and the suffix ``_template``.
    """
    try:
        tag_name, instance_name = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError("%r tag requires exactly two arguments" % 
                                  token.contents.split()[0])
    
    node = ModelTeaserNode(instance_name, resolved=False)
    return node
    

@register.tag
def list_block(parser, token):
    """
    Template tag that takes a model instance manager and returns the given 
    instances as a list in a template-formatted block.  Inserts two objects 
    into the context:
      ``model`` - The name of the model for the list
      ``instance_list`` - An iterable of model instances.
    
    You would want to use the tag instead of the filter primarily because, with
    a tag you can override the default template used for a particular model.
    For example, in the example project, Pepulators have Jambs, so when we put
    a detail block of a Pepulator on a page, a list of Jambs will show up.  Say
    we don't want the default detail of jambs to display; say we want a custom
    Jamb detail template.  Then we can say the following:
    
        {% with pepulator_factory_jamb_template="pepulator_factory/jamb_detail.html" %}
            {% detail_block pepulator %}
        {% endwith %}
    
    The custom template is named by the app name (``pepulator_factory``), the 
    model name in all lowercase (``jamb``) and the suffix ``_template``.
    """
    try:
        tag_name, list_name = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError("%r tag requires exactly two arguments" % 
                                  token.contents.split()[0])
    
    node = ModelListNode(list_name, resolved=False)
    return node




from django.db.models.manager import Manager
from django.template import Context, Node, Variable
from django.template.loader import get_template

class BaseModelBlockNode (Node):
    def __init__(self, thing, resolved=True):
        """
        thing -- The thing (probably a model instance or a list of model 
                 instances) to be rendered as a block.
        resolved -- If True, then ``thing`` is a resolved value.  If False,
                    then thing is the name of a variable which, in context,
                    will contain the value of the thing.
        """
        self.thing = thing
        self.resolved = resolved
    
    def get_template_variable(self, thing, type_of_thing):
        """
        Return the name of the template variable that should be used to render
        the thing.  If the variable name does not resolve to a value, then a 
        default template will be used.
        """
        if isinstance(thing, (list, tuple)):
            template_variable = ''
        elif hasattr(thing, 'model') and thing:
            template_variable = '%s_%s_%s_template' % \
                (thing.model._meta.app_label, thing.model._meta.module_name, 
                 type_of_thing)
        elif hasattr(thing, '_meta') and thing:
            template_variable = '%s_%s_%s_template' % \
                (thing._meta.app_label, thing._meta.module_name, type_of_thing)
        else:
            template_variable = ''
        return template_variable
    
    def get_resolved_value(self, context):
        """
        Return a resolved version of the thing being rendered -- either a model
        instance or a list of such instances. Specifically, if the name of the
        value was passed to the node as a string, resolve the value w.r.t. the
        context. If the actual value was passed in, then just return the value. 
        """
        if not self.resolved:
            res_var = Variable(self.thing).resolve(context)
        else:
            res_var = self.thing
        return res_var
    
    def render(self, context):
        # Grab the thing to render.  It'll be a model instance, or a queryset,
        # or a list or something.
        thing = self.get_resolved_value(context)
        
        # Get the name of the context variable that holds the override template
        # file name.
        template_variable = self.get_template_variable(thing, self.thing_type)
        
        # If the override template file name is set, then use that.  Otherwise,
        # use the default template for the thing type
        template_name = context.get(template_variable,
                                    'model_blocks/object_%s.html' % self.thing_type)
        template = get_template(template_name)
        
        # Get the data necessary for rendering the thing, and add it to the
        # context.
        context.update(Context(self.get_context_data(thing, context)))
        
        # The variable 'title' must be in the context.  If it's not there, add
        # it and set it to None
        if 'title' not in context:
            context['title'] = None
        
        # After rendering, pop off of the context so that it's back to normal 
        # for the next tag.
        rendering = template.render(context)
        context.pop()
        return rendering


class ModelDetailNode (BaseModelBlockNode):
    
    thing_type = 'detail'
    
    def __get_fields_list_variable(self, instance, var_suffix):
        # The variable name is built up from the name of the app, the name of 
        # the model, and the specified suffix.
        fields_list_variable = '%s_%s_%s' % \
            (instance._meta.app_label, instance._meta.module_name, var_suffix)
        
        return fields_list_variable
        
    
    def __get_field_list(self, instance, context, var_suffix):
        # Get the variable name to look for
        var = self.__get_fields_list_variable(instance, var_suffix)
        fields_str = context.get(var, None)
        
        # If the variable is not set, just return an empty list
        if fields_str is None:
            return []
        
        # If the variable is set, split the list on comma (,) and return
        else:
            include_fields = [field.strip() for field in fields_str.split(',')]
            return include_fields
            
    
    def get_include_fields(self, instance, context):
        """Return a list of fields and order to include in the rendering."""
        return self.__get_field_list(instance, context, 'fields')
        
    
    def get_exclude_fields(self, instance, context):
        """Return a list of fields to exclude from the rendering."""
        return self.__get_field_list(instance, context, 'exclude')
        
    
    def get_context_data(self, instance, context):
        """
        Calculate additional context data that will be used to render the thing.
        
        """
        include_fields = self.get_include_fields(instance, context)
        exclude_fields = self.get_exclude_fields(instance, context)
        
        fields = []
        for field in instance._meta.fields:
            name = field.name
            
            if name in exclude_fields:
                continue
            
            if include_fields and name not in include_fields:
                continue
            
            label = field.verbose_name
            value = getattr(instance, field.name)
            is_list = False
            is_link = (type(field).__name__ in ('URLField',))
            model = instance._meta.module_name
            
            if value is not None:
                fields.append((
                    name, label, value, is_list, is_link, model,
                ))
        
        for rel_obj, model in instance._meta.get_all_related_objects_with_model():
            name = rel_obj.get_accessor_name()
            
            if name in exclude_fields:
                continue
            
            if include_fields and name not in include_fields:
                continue
            
            label = name
            value = getattr(instance, name)
            is_list = isinstance(value, (list, tuple, Manager))
            is_link = False
            
            if value is not None:
                fields.append((
                    name, label, value, is_list, is_link, model,
                ))
        
        # If include_fields was defined, then sort by the order.
        if include_fields:
            fields = sorted(fields, key=lambda field: include_fields.index(field[0]))
                
        
        return {'model':instance._meta.module_name, 
                'instance':instance, 
                'fields':fields}


class ModelTeaserNode (ModelDetailNode):
    
    thing_type = 'teaser'


class ModelListNode (BaseModelBlockNode):
    
    thing_type = 'list'
    
    def get_context_data(self, queryset, context):
        if hasattr(queryset, 'model') and queryset.model:
            model = queryset.model._meta.module_name
        else:
            model = None
        
        return {'model':model, 'instance_list':queryset}
    




from django.template import Context, Library
from django.template.loader import get_template

from model_nodes import ModelDetailNode, ModelTeaserNode, ModelListNode

register = Library()

@register.filter
def as_detail_block(instance, title=None):
    """
    Template filter that returns the given instance as a template-formatted
    block.  Inserts two objects into the context:
      ``instance`` - The model instance
      ``fields`` - A list of (name, label, value) tuples representing the 
                   instance's fields
    """
    node = ModelDetailNode(instance)
    return node.render(Context({'title':title}))


@register.filter
def as_teaser_block(instance, title=None):
    """
    Template filter that returns the given instance as a template-formatted
    teaser.  Inserts two objects into the context:
      ``instance`` - The model instance
      ``fields`` - A list of (name, label, value) tuples representing the 
                   instance's fields
    """
    node = ModelTeaserNode(instance)
    return node.render(Context({'title':title}))


@register.filter
def as_list_block(queryset, list_title=None):
    """
    Template filter that returns the given instance list as a template-formatted
    block.  Inserts into the context:
        ``instance_list`` - The list of instances
    """
    node = ModelListNode(queryset)
    return node.render(Context({'title':list_title}))
    



"""
Test the model blocks
"""

import datetime

from django.test import TestCase
from mock import Mock

from django.db.models import Model, IntegerField, DateTimeField, CharField
from django.template import Context, Template, TemplateSyntaxError

from example_project.pepulator_factory.models import Pepulator, Distributor
from model_blocks.templatetags import model_filters
from model_blocks.templatetags import model_nodes

class DetailBlockFilterTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template(('{{ title|default_if_none:instance|safe }}:{{ model|safe }},'
                                   '{% for name, label, value, is_list, is_link in fields %}'
                                     '{{ name|safe }},'
                                     '{{ label|safe }},'
                                     '{% if not is_list %}'
                                       '{% if is_link %}'
                                         '@{{ value }}'
                                       '{% else %}'
                                         '{{ value|safe }}'
                                       '{% endif %}'
                                     '{% else %}'
                                       '[{% for item in value.all %}{{ item|safe }},{% endfor %}]'
                                     '{% endif %},'
                                   '{% endfor %}')))
    
    
    def test_model_format(self):
        """Tests that a given model is formatted as expected."""
        pepulator = Pepulator.objects.get(serial_number=1235)
        
        expected_detail = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,@ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        detail = model_filters.as_detail_block(pepulator)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_detail.html')
        self.assertEqual(detail, expected_detail)
    
    
    def test_filter_is_registered(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulator|as_detail_block }}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_detail = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,@ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        detail = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_detail.html')
        self.assertEqual(detail, expected_detail)
    
    
    def test_title_is_used(self):
        """Test that a title is used if provided"""
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulator|as_detail_block:"My Pepulator" }}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_detail = (u"My Pepulator:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,@ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        detail = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_detail.html')
        self.assertEqual(detail, expected_detail)


    def test_related_fields(self):
        """Tests that related fields not defined on the model are included."""
        pepulator = Distributor.objects.get(name="Mom & Pop")
        
        expected_detail = (u"Mom & Pop:distributor,"
            "name,name,Mom & Pop,"
            "capacity,capacity,175,"
            "stock,stock,[Pepulator #1238,],"
        )
        detail = model_filters.as_detail_block(pepulator)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_detail.html')
        self.assertEqual(detail, expected_detail)
    
    
class TeaserBlockFilterTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:instance|safe }}:{{ model|safe }},{% for name, label, value, is_list in fields %}{{ name|safe }},{{ label|safe }},{% if not is_list %}{{ value|safe }}{% else %}[{% for item in value.all %}{{ item|safe }},{% endfor %}]{% endif %},{% endfor %}'))
    
    
    def test_model_format(self):
        """Tests that a given model is formatted as expected."""
        pepulator = Pepulator.objects.get(serial_number=1235)
        
        expected_teaser = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        teaser = model_filters.as_teaser_block(pepulator)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_teaser.html')
        self.assertEqual(teaser, expected_teaser)
    
    
class ListBlockFilterTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:model|capfirst }}{% if not title %}s{% endif %}:{{ instance_list|safe }}'))
    
    
    def test_list_format(self):
        """Tests that a given model is formatted as expected."""
        pepulator_list = Pepulator.objects.filter(serial_number__gt=2000)
        
        expected_rendering = (u"Pepulators:[<Pepulator: Pepulator #2345>, "
                               "<Pepulator: Pepulator #2346>]")
        rendering = model_filters.as_list_block(pepulator_list)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)
    
    
    def test_filter_is_registered(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulators|as_list_block }}'))
        pepulator_list = Pepulator.objects.filter(serial_number__gt=2000)
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Pepulators:[<Pepulator: Pepulator #2345>, "
                               "<Pepulator: Pepulator #2346>]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)

    
    def test_empty_queryset(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulators|as_list_block }}'))
        pepulator_list = Pepulator.objects.filter(serial_number__gt=5000)
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Pepulators:[]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)

    
    def test_non_query_set_results_in_no_model(self):
        """Test that when a non queryset is used, the model is None"""
        # Why? Because we try to read the model off of the queryset. If we just
        # have a list of objects, then we don't know the model.
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulators|as_list_block }}'))
        pepulator_list = [p for p in Pepulator.objects.filter(serial_number__gt=2000)]
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Nones:[<Pepulator: Pepulator #2345>, "
                               "<Pepulator: Pepulator #2346>]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)

    
    def test_empty_list(self):
        """Test that when a non queryset is used, the model is None"""
        
        template = Template(('{% load model_filters %}'
                             '{{ pepulators|as_list_block }}'))
        pepulator_list = []
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Nones:[]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)

    
    def test_alternate_title_is_used(self):
        """Test that a list title is used if provided"""
        template = Template(('{% load model_filters %}'
                             '{{ pepulators|as_list_block:"Some Pepulators" }}'))
        pepulator_list = Pepulator.objects.filter(serial_number__gt=2000)
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Some Pepulators:[<Pepulator: Pepulator #2345>, "
                               "<Pepulator: Pepulator #2346>]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('model_blocks/object_list.html')
        self.assertEqual(rendering, expected_rendering)


class DetailBlockTagTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:instance|safe }}:{{ model|safe }},{% for name, label, value, is_list in fields %}{{ name|safe }},{{ label|safe }},{% if not is_list %}{{ value|safe }}{% else %}[{% for item in value.all %}{{ item|safe }},{% endfor %}]{% endif %},{% endfor %}'))
    
    
    def test_tag_is_registered(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_tags %}'
                             '{% with pepulator_factory_pepulator_detail_template="pepulator_factory/pepulator_detail.html" %}'
                             '{% detail_block pepulator %}'
                             '{% endwith %}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_detail = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        detail = template.render(context)
        
        model_nodes.get_template.assert_called_with('pepulator_factory/pepulator_detail.html')
        self.assertEqual(detail, expected_detail)
    
    
    def test_with_specific_fields(self):
        """Test that the included fields spec is respected"""
        
        template = Template(('{% load model_tags %}'
                             '{% with pepulator_factory_pepulator_detail_template="pepulator_factory/pepulator_detail.html" %}'
                             '{% with pepulator_factory_pepulator_fields="serial_number, color, height, width" %}'
                             '{% detail_block pepulator %}'
                             '{% endwith %}'
                             '{% endwith %}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_detail = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "color,color,red,"
            "height,height,12,"
            "width,width,15,"
        )
        detail = template.render(context)
        
        self.assertEqual(detail, expected_detail)
    
    
    def test_with_excluded_fields(self):
        """Test that the excluded fields spec is respected"""
        
        template = Template(('{% load model_tags %}'
                             '{% with pepulator_factory_pepulator_detail_template="pepulator_factory/pepulator_detail.html" %}'
                             '{% with pepulator_factory_pepulator_exclude="knuckles, jambs, color, address" %}'
                             '{% detail_block pepulator %}'
                             '{% endwith %}'
                             '{% endwith %}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_detail = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "distributed_by,distributed by,Walmart,"
        )
        detail = template.render(context)
        
        self.assertEqual(detail, expected_detail)
    
    
    def test_fail_on_wrong_number_of_arguments(self):
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% detail_block pepulator "overflow" %}'))
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% detail_block %}'))
    
    
class TeaserBlockTagTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:instance|safe }}:{{ model|safe }},{% for name, label, value, is_list in fields %}{{ name|safe }},{{ label|safe }},{% if not is_list %}{{ value|safe }}{% else %}[{% for item in value.all %}{{ item|safe }},{% endfor %}]{% endif %},{% endfor %}'))
    
    
    def test_tag_is_registered(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_tags %}'
                             '{% with pepulator_factory_pepulator_teaser_template="pepulator_factory/pepulator_teaser.html" %}'
                             '{% teaser_block pepulator %}'
                             '{% endwith %}'))
        
        pepulator = Pepulator.objects.get(serial_number=1235)
        context = Context({'pepulator':pepulator})
        
        expected_teaser = (u"Pepulator #1235:pepulator,"
            "serial_number,serial number,1235,"
            "height,height,12,"
            "width,width,15,"
            "manufacture_date,manufacture date,2011-06-10 11:12:33,"
            "color,color,red,"
            "address,address,ppr://1235/,"
            "distributed_by,distributed by,Walmart,"
            "knuckles,knuckles,[Knuckle of hardness 2.35,Knuckle of hardness 1.10,],"
            "jambs,jambs,[],"
        )
        teaser = template.render(context)
        
        model_nodes.get_template.assert_called_with('pepulator_factory/pepulator_teaser.html')
        self.assertEqual(teaser, expected_teaser)
    
    def test_fail_on_wrong_number_of_arguments(self):
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% teaser_block pepulator "overflow" %}'))
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% teaser_block %}'))
    
    
class ListBlockTagTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:model|capfirst }}{% if not title %}s{% endif %}:{{ instance_list|safe }}'))
    
    
    def test_filter_is_registered(self):
        """Test that the filter can be used from within a template"""
        
        template = Template(('{% load model_tags %}'
                             '{% with pepulator_factory_pepulator_list_template="pepulator_factory/pepulator_list.html" %}'
                             '{% list_block pepulators %}'
                             '{% endwith %}'))
        pepulator_list = Pepulator.objects.filter(serial_number__gt=2000)
        context = Context({'pepulators':pepulator_list})
        
        expected_rendering = (u"Pepulators:[<Pepulator: Pepulator #2345>, "
                               "<Pepulator: Pepulator #2346>]")
        rendering = template.render(context)
        
        model_nodes.get_template.assert_called_with('pepulator_factory/pepulator_list.html')
        self.assertEqual(rendering, expected_rendering)
    
    def test_fail_on_wrong_number_of_arguments(self):
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% list_block pepulators "overflow" %}'))
        self.assertRaises(TemplateSyntaxError, Template, 
                          ('{% load model_tags %}'
                           '{% list_block %}'))

class ModelBlockModuleTest (TestCase):
    def test_all_tags_and_filters_loaded(self):
        template = Template(('{% load model_blocks %}'
                             '{% detail_block pepulator %}'
                             '{% list_block pepulators %}'
                             '{{ pepulator|as_detail_block }}'
                             '{{ pepulators|as_list_block }}'))
                             
        # We just care that everything loaded, and we were able to get here
        # without incidence.
        self.assert_(True)


class SideEffectsTest (TestCase):
    fixtures = ['pepulator_factory_data.json']

    def setUp(self):
        # Mock Django's get_template so that it doesn't load a real file;
        # instead just return a template that allows us to verify the context
        model_nodes.get_template = Mock(
            return_value=Template('{{ title|default_if_none:model|capfirst }}{% if not title %}s{% endif %}'))
    
    
    def test_model_doesnt_carry_over_into_future_blocks(self):
        template = Template(('{% load model_tags %}'
                             '{{ model }}'
                             '{% list_block distributors %}'
                             '{{ model }}'))
        distributor_list = Distributor.objects.all()
        context = Context({'model':'My String',
                           'distributors':distributor_list})
        
        expected_rendering = (u"My String"
                               "Distributors"
                               "My String")
        rendering = template.render(context)
        
        self.assertEqual(rendering, expected_rendering)




from django.template import Library

from model_filters import as_detail_block, as_teaser_block, as_list_block
from model_tags import detail_block, teaser_block, list_block

register = Library()

register.tag(detail_block)
register.tag(teaser_block)
register.tag(list_block)
register.filter(as_detail_block)
register.filter(as_teaser_block)
register.filter(as_list_block)




from setuptools import setup, find_packages

model_blocks = __import__('model_blocks')

readme_file = 'README.rst'
try:
    long_description = open(readme_file).read()
except IOError, err:
    sys.stderr.write("[ERROR] Cannot find file specified as "
        "``long_description`` (%s)\n" % readme_file)
    sys.exit(1)

setup(name='django-model-blocks',
      version='0.8.10',
      description=('Simple filters and tags for generic Django '
                   'model template partials'),
      long_description=long_description,
      zip_safe=False,
      author='Mjumbe Wawatu Ukweli',
      author_email='mjumbewu@kwawatu.com',
      url='https://github.com/mjumbewu/django-model-blocks/',
      download_url='https://github.com/mjumbewu/django-model-blocks/downloads',
      packages = find_packages(exclude=['example_project', 'example_project.*']),
      include_package_data=True,
      install_requires = [
        'Django>=1.2.1',
      ],
      obsoletes = [
        'model_filters',
      ],
      provides = [
        'model_blocks',
      ],
      classifiers = ['Development Status :: 4 - Beta',
                     'Environment :: Web Environment',
                     'Framework :: Django',
                     'Intended Audience :: Developers',
                     'License :: OSI Approved :: BSD License',
                     'Operating System :: OS Independent',
                     'Programming Language :: Python',
                     'Topic :: Utilities'],
      )
